//----------------------------------------------------------------------
// GVID.H                         Video Libraries by Ryan Geiss (c) 1997
//----------------------------------------------------------------------


#pragma inline                          // tells compiler to use TASM rather than the built-in BASM.



// NOTES on stream i/o, etc.
//
// To read in just one char, w/o linefeed:   a = getch()
// To read in a char, requiring a linefeed:  a = getchar()                      (macro of the one below)
//                                                                                                                      a = getc(stdin);
//


#ifndef GEISS_VIDCORE
#define GEISS_VIDCORE 1



#include "malloc.h"     // for virtual screen memory allocation
#include "stdio.h"      // for writeTGA
#include "stdlib.h"     // for exit() function
#include "conio.h"      // for kbhit() function


#define COLOR_MASK         0x3C6        // the bit mask register
#define COLOR_REGISTER_RD  0x3C7        // set read index at this I/O
#define COLOR_REGISTER_WR  0x3C8        // set write index at this I/O
#define COLOR_DATA         0x3C9        // the R/W data is here




unsigned char far *VS1 = NULL;          // pointer to virtual screen.
unsigned char far *VS2 = NULL;          // pointer to virtual screen.
unsigned char far *tempptr = NULL;      // temp ptr. for Buffer_Swap()

FILE *outfile;



 

/* 
   use VS1 = farmalloc(64000) in your main loop to initialize a buffer.
   
   random numbers:    srand(time(NULL));  (requires time.h + stdlib.h)
		      k = rand() % 100;   (requires stdlib.h)
		      *** works on seconds, not milliseconds!!!

   time:     time_t starttime, endtime;
	     time(&starttime);
	     time(&endtime);
	     diff_time(starttime, endtime) returns seconds as int

*/


//----------------------------------------------------------------------
// PROTOTYPES
//----------------------------------------------------------------------


void  putpixel(int x, int y, unsigned char c);
unsigned char getpixel(int x, int y);
unsigned char getsubpixel(unsigned char far *buffer, float x, float y);

void waitkey();
void set13(void);
void setfunky(unsigned char x);
void settext(void);
void wait_refresh(void);

void setRGB(unsigned char co,unsigned char r,unsigned char g,unsigned char b);
void getRGB(unsigned char co,unsigned char *,unsigned char *,unsigned char *);

void Buffer_Paste(unsigned char far *buffer);
void Buffer_Clear(unsigned char far *buffer);
void Buffer_Copy(unsigned char far *buffer1, unsigned char far *buffer2);
void Buffer_Swap(unsigned char far *buffer1, unsigned char far *buffer2);

void open_TGA(int fr, int x_span, int y_span);
void plot_TGA(float b, float g, float r);
void close_TGA();
void put_TGA_to_buffer(char *s, unsigned char far *buffer);



//----------------------------------------------------------------------
// BODIES
//----------------------------------------------------------------------

void putpixel(int x, int y, unsigned char c)
{
   asm 
   {

       mov      bx, 40960
		 mov       ax, 320
		 mul       word ptr y
       add       ax, x
       mov       di, ax
       mov      es, bx

       mov       al, c

       stosb                             // moves al to es:di

   }
}

unsigned char getpixel(int x, int y)
{
   unsigned char c;
   asm 
   {
       push ds                           // BASM automatically does this;
					 // with TASM you must preserve ds.
      ;mov       ax, 40960
      ;mov       ds, ax
      ;mov       bx, y
      ;shl       bx, 2
      ;add       bx, y
      ;shl       bx, 6
      ;add       bx, x
      ;mov       si, bx
      ;lodsb                             // moves [ds:si] to al
      ;mov       c, al

       mov       cx, y
       mov      ax, 40960
       mov       bx, cx
       mov      ds, ax
       shl       bx, 2
       add       bx, cx
       shl       bx, 6
       add       bx, x
       mov       si, bx
       lodsb                             // moves [ds:si] to al
       mov       c, al

       pop ds
   }
   return c;
}

unsigned char getsubpixel(unsigned char far *buffer, float x, float y)
{
   int x2, y2;
   static float dx, dy, dx_invert, dy_invert, e;
   unsigned char a, b, c, d;
   
   x2 = x;
   y2 = y;
   dx = x - (float)x2;          // 0..1
   dy = y - (float)y2;          // 0..1
	dx_invert = 1-dx;
	dy_invert = 1-dy;

	//printf("float (%7.5f,%7.5f) int (%d,%d) dx=%7.5f, dy=%7.5f\n", x,y,x2,y2,dx, dy);

   asm 
   {
       push ds                           // BASM automatically does this;
					 // with TASM you must preserve ds.
	    lds si, buffer

		 mov       ax, 320
		 mul       word ptr y2
       add       ax, x2
       mov       si, ax

		 add       si, 4   ;// crackhead patch alert!

       lodsb                             // moves [ds:si] to al (1 byte) & inc. si
       mov       a, al

	 
       lodsb                             // moves [ds:si] to al
       mov       b, al
		
		 add       si, 318
       lodsb                             // moves [ds:si] to al
       mov       c, al

		 
       lodsb                             // moves [ds:si] to al
       mov       d, al

       pop ds
   }
   
//   a = getpixel(x2, y2);
//   b = getpixel(x2+1, y2);
//   c = getpixel(x2, y2+1);
//   d = getpixel(x2+1, y2+1);

   e =  a*dx_invert*dy_invert; 
   e += c*dx_invert*dy;
   e += b*dy_invert*dx; 
   e += d*dx*dy;

	//return getpixel((int)x+1, (int)y+1);
   return (unsigned char)e;
}



void waitkey()
{
	asm 
	{
		mov   ah, 1                                                             //DOS: get a character (puts it in AL)
		int     21h
	}
}

void set13()
{
   asm 
   {
      mov       ax, 0013h;              // set video mode
      int       10h;                    // to 320x200x256
   }
}

void setfunky(unsigned char x)
{
	asm 
	{
     MOV AX,0013H; INT 10H;
     mov dx,3d4h
     mov al,9
     out dx,al
     inc dx
     in al,dx                  // 0 hi/dbl,  1 norm,    2 320x100, 4 320x75  }
     and al,0e0h               // 5 320x66,  6 320x57,  7 320x50,  8 320x44  }
     add al, byte ptr x        // 9 320x39,  10 320x36, 11 320x33, 12 320x30 }
     out dx,al                 // 13 320x28, 14 320x26, 15 320x24, 16 320x23 }
	}                                                                        // 20 320x18, 24 320x15, 28 320x13, 31 320x12 }
}                              

void settext()
{
   asm
   {
      mov       ax, 0003h;
      int       10h;
   }   
}

void wait_refresh()
{
	asm 
	{
			mov     dx,3dah         //wait for vertical retrace
		NoVrt:
		   in      al,dx
		   test    al,8          //Test 3rd bit again
		   jz      NoVrt            //Wait until Verticle Retrace ends
	}
}


   
void setRGB(unsigned char co,unsigned char r,unsigned char g,unsigned char b)
{
   asm
   {
      mov   al, co

      mov   dx, COLOR_REGISTER_WR
      cli                                       // disable interrupts
      out   dx, al


      mov   dx, COLOR_DATA

      mov   al, r
      out   dx, al

      mov   al, g
      out   dx, al

      mov   al, b
      out   dx, al

      sti                                       // re-enable interrupts
   }

   //outp(COLOR_REGISTER_WR, co);
   //outp(COLOR_DATA, r);
   //outp(COLOR_DATA, g);
   //outp(COLOR_DATA, b);
}

void getRGB(unsigned char co,unsigned char *r,unsigned char *g,unsigned char *b)
{
   asm
   {
      mov   al, co

      mov   dx, COLOR_REGISTER_RD
      cli
      out   dx, al


      mov   dx, COLOR_DATA

      in    al, dx
      mov   bx, r
      mov   [bx], al   // if this won't compile use BYTE PTR [bx].

      in    al, dx
      mov   bx, g
      mov   [bx], al

      in    al, dx
      mov   bx, b
      mov   [bx], al

      sti
   }

   //outp(COLOR_REGISTER_RD, co);
   //outp(COLOR_DATA, *r);
   //outp(COLOR_DATA, *g);
   //outp(COLOR_DATA, *b);
}


void Buffer_Paste(unsigned char far *buffer)
{
  asm                   // "rep movsd" moves ecx dwords from ds:si to es:di
  {
    push ds;
    mov ax, 40960;              
    mov di, 0;                  // es:di is the dest of memory to move to
    mov ecx, 16000;             // size of buffer in DWORDS (# of reps)
    mov es, ax;                 
    lds si, buffer;             // ds:si is the source
    rep movsd;                  // move 16000 dwords (64000 bytes)

    pop ds;   
  }  
}




void Buffer_Clear(unsigned char far *buffer)
{
  asm                   // "rep movsd" moves ecx dwords from ds:si to es:di
  {
    push ds;
    mov ecx, 16000;             // size of buffer in DWORDS (# of reps)
    mov eax, 0;                                          // THIS WAS CHANGED - USED TO BE EAX!
    les di, buffer;             // es:di is the dest
    rep stosd;                  // move 16000 dwords (64000 bytes)
    pop ds;   
  }
}



void Buffer_Copy(unsigned char far *buffer1, unsigned char far *buffer2)
{                                                               // copies second buffer to first buffer.
  asm                   // "rep movsd" moves ecx dwords from ds:si to es:di
  {
    push ds;
    mov ecx, 16000;             // size of buffer in DWORDS (# of reps)
    les di, buffer1;                              // es:di is the dest
	 lds si, buffer2;            // ds:si is the source
    rep movsd;                  // move 16000 dwords (64000 bytes)
    pop ds;   
  }  
}

void Buffer_Swap(unsigned char far *buffer1, unsigned char far *buffer2)
{
	// doesn't work; ptr values swap here but not back in the main program.
	tempptr = buffer1;
	buffer1 = buffer2;
	buffer2 = tempptr;

	tempptr = NULL;
}


void open_TGA(int fr, int x_span, int y_span)
{
  // looks to the arrays VS_R[], VS_G[], VS_B[] to write 
  // a 24-bit dynamically-sized TGA.  Below is special fix for VidEdit
  // (...VidEdit requires at least 320x200 image size)
//              if (xneg>0) xneg=0;
//              if (xpos<319) xpos=319;
//              if (yneg>0) yneg=0;
//              if (ypos<199) ypos=199;

  char filename[13] = "xxxxxxxxxxxx\0";
  sprintf(filename, "data%04d.tga", fr);
  outfile=fopen(filename, "wb");

  fprintf(outfile, "%c%c%c%c%c%c",  0,  0,  2,  0,  0,  0);
  fprintf(outfile, "%c%c%c%c%c%c",  0,  0,  0,  0,  0,  0);
  fprintf(outfile, "%c%c%c%c%c%c", x_span % 256, x_span/256, y_span % 256, y_span/256, 24, 32);
}


void plot_TGA(float b, float g, float r)
{
  // requires (screen.width*screen.height) calls per frame!

  static unsigned char x,y,z;

  x = (unsigned char)(b*255);
  y = (unsigned char)(g*255);
  z = (unsigned char)(r*255);
  fprintf(outfile, "%c%c%c", x, y, z);
}


void close_TGA()
{
  fclose(outfile);
}


void put_TGA_to_buffer(char *s, unsigned char far *buffer)
{
  // looks to the arrays VS_R[], VS_G[], VS_B[] to write 
  // a 24-bit dynamically-sized TGA.  Below is special fix for VidEdit
  // (...VidEdit requires at least 320x200 image size)
//              if (xneg>0) xneg=0;
//              if (xpos<319) xpos=319;
//              if (yneg>0) yneg=0;
//              if (ypos<199) ypos=199;

  FILE *infile;
  int x_span, y_span;
  unsigned char a,b,c,d,e,f;
  unsigned int x, y, offset;

  infile = fopen(s, "r");
  fscanf(infile, "%c%c%c%c%c%c", &a,&b,&c,&d,&e,&f);
  //printf("%u-%u-%u-%u-%u-%u\n", a,b,c,d,e,f);
  fscanf(infile, "%c%c%c%c%c%c", &a,&b,&c,&d,&e,&f);
  //printf("%u-%u-%u-%u-%u-%u\n", a,b,c,d,e,f);
  fscanf(infile, "%c%c%c%c%c%c", &a,&b,&c,&d,&e,&f);
  //printf("%u-%u-%u-%u-%u-%u\n", a,b,c,d,e,f);
  x_span = a+b*256;
  y_span = c+d*256;

  offset = ((200-y_span)/2)*320 + (320-x_span)/2;
  //printf("Image: %ux%u\n", x_span, y_span);

  for (x=0; x<256*3; x++) a = fgetc(infile);


  for (y=y_span; y>0; y--)
    for (x=0; x<x_span; x++)
	 {
		//fscanf(infile, "%c", &a);
		a = fgetc(infile);
		//printf("%d ", a);
		if (a<250)
			buffer[y*320+x + offset] = 63-a/4;
	 }

  //printf("Done!\n");

  fclose(infile);
}



#endif



